#include "sht.h"


#include <stdlib.h>

static unsigned long h1 ( char * k ){      
     /* ROLE         Returns polynomial hash of a string key 
        RETURNS      this hash value still needs to be scaled
        PARAMETERS   k - string used as key 
     */     
     
     // Magic polynomial constant 
     unsigned long a = 33; // 33, 37, 39, 41
     
     int length = _HASH_KEY_STRL_; 
     
     // Array used to store temporarily the string decoding
     unsigned long values[_HASH_KEY_STRL_]; 
       
     // For each element of values
     int n = 0;  
     
     // let's decode characters into unsigned integers
     while ( n < length ){
           if (k[n])     values[n] = (unsigned int)k[n]; 
           else          break;
           n++;
     }
       
     // let's fill what's left of the _HASH_KEY_SIZE_ characters with 0
     while ( n < length ){ 
           values[n] = 0L; 
           n++;      
     }
       
     // Result is a polynomial in a
     unsigned long result = 0L;
     for(n = 0 ; n < length ; n++){
           result = values[n] + a * result; 
     }
       
     return result;
}



static int h2 ( unsigned long n){
     /* ROLE         This Compression Function scales the values h1 returns 
                     so that they are usable as indexes in our buckets array
        RETURNS      Scaled down version of n
        PARAMETERS   n - h1(key)
     */     
     
    return (int)(n% _HASH_SIZE_); 
}

int hashFunction(char* k){
    /* ROLE         This Compression Function scales the values h1 returns 
                     so that they are usable as indexes in our buckets array
       RETURNS      Scaled down version of n
       PARAMETERS   n - h1(key)
    */
     
    // do the hashing 
    int n = h2(h1(k));    
    
    
    #ifdef _DEBUG_MODE_
    // let's intercept out-of-bound errors 
    if ((n<0) || (n>=_HASH_SIZE_)){ 
              printf("ERROR\thashfunction\n");
              printf("\thash value out of boundaries\n"); 
              printf("\tvalue is %d\n", n); 
              exit(EXIT_FAILURE); 
    }
    #endif 
    
    return n;
}


